@@ -31,6 +31,7 @@
 (declare-function message-buffers "message.el")
 (declare-function addressbook-set-mail-buffer-1 "ext:addressbook-bookmark.el"
                   (&optional bookmark-name append cc))
+(declare-function addressbook-bookmark-set-1 "ext:addressbook-bookmark.el" (&optional contact))
 (declare-function helm-browse-project "helm-files" (arg))
 
 
@@ -44,15 +45,14 @@
   :type 'boolean)
 
 (defcustom helm-bookmark-default-filtered-sources
-  (append '(helm-source-bookmark-files&dirs
+  (append '(helm-source-bookmark-org
+            helm-source-bookmark-files&dirs
             helm-source-bookmark-helm-find-files
             helm-source-bookmark-info
             helm-source-bookmark-gnus
             helm-source-bookmark-man
             helm-source-bookmark-images
             helm-source-bookmark-w3m)
-          (and (locate-library "addressbook-bookmark")
-               (list 'helm-source-bookmark-addressbook))
           (list 'helm-source-bookmark-uncategorized
                 'helm-source-bookmark-set))
   "List of sources to use in `helm-filtered-bookmarks'."
@@ -142,6 +142,11 @@
   "Face used for file bookmarks."
   :group 'helm-bookmark)
 
+(defface helm-bookmark-file-not-found
+    '((t (:foreground "Slategray4")))
+  "Face used for file bookmarks."
+  :group 'helm-bookmark)
+
 (defface helm-bookmark-directory
     '((t (:inherit helm-ff-directory)))
   "Face used for file bookmarks."
@@ -286,8 +291,8 @@ BOOKMARK is a bookmark name or a bookmark record."
 (defun helm-bookmark-image-bookmark-p (bookmark)
   "Return non-nil if BOOKMARK bookmarks an image file."
   (if (stringp bookmark)
-      (assoc 'image-type (assoc bookmark bookmark-alist))
-    (assoc 'image-type bookmark)))
+      (assq 'image-type (assq bookmark bookmark-alist))
+    (assq 'image-type bookmark)))
 
 (defun helm-bookmark-file-p (bookmark)
   "Return non-nil if BOOKMARK bookmarks a file or directory.
@@ -297,6 +302,11 @@ This excludes bookmarks of a more specific kind (Info, Gnus, and W3m)."
          (isnonfile  (equal filename helm-bookmark--non-file-filename))) 
     (and filename (not isnonfile) (not (bookmark-get-handler bookmark)))))
 
+(defun helm-bookmark-org-file-p (bookmark)
+  (let* ((filename (bookmark-get-filename bookmark)))
+    (or (string-suffix-p ".org" filename t)
+        (string-suffix-p ".org_archive" filename t))))
+
 (defun helm-bookmark-helm-find-files-p (bookmark)
   "Return non-nil if BOOKMARK bookmarks a `helm-find-files' session.
 BOOKMARK is a bookmark name or a bookmark record."
@@ -312,7 +322,8 @@ BOOKMARK is a bookmark name or a bookmark record."
 
 (defun helm-bookmark-uncategorized-bookmark-p (bookmark)
   "Return non--nil if BOOKMARK match no known category."
-  (cl-loop for pred in '(helm-bookmark-addressbook-p
+  (cl-loop for pred in '(helm-bookmark-org-file-p
+                         helm-bookmark-addressbook-p
                          helm-bookmark-gnus-bookmark-p
                          helm-bookmark-w3m-bookmark-p
                          helm-bookmark-woman-man-bookmark-p
@@ -413,6 +424,19 @@ than `w3m-browse-url' use it."
             (helm-init-candidates-in-buffer
                 'global (helm-bookmark-man-setup-alist)))))
 
+;;; Org files
+;;
+(defun helm-bookmark-org-setup-alist ()
+  "Specialized filter function for Org file bookmarks."
+  (helm-bookmark-filter-setup-alist 'helm-bookmark-org-file-p))
+
+(defvar helm-source-bookmark-org
+  (helm-make-source " Bookmarked Org files" 'helm-source-filtered-bookmarks
+    :init (lambda ()
+            (bookmark-maybe-load-default-file)
+            (helm-init-candidates-in-buffer
+                'global (helm-bookmark-org-setup-alist)))))
+
 ;;; Gnus
 ;;
 (defun helm-bookmark-gnus-setup-alist ()
@@ -591,6 +615,16 @@ than `w3m-browse-url' use it."
 (defvar helm-source-bookmark-addressbook
   (helm-make-source "Bookmark Addressbook" 'helm-bookmark-addressbook-class))
 
+(defvar helm-source-addressbook-set
+  (helm-build-dummy-source "Addressbook add contact"
+    :filtered-candidate-transformer
+    (lambda (_candidates _source)
+      (list (or (and (not (string= helm-pattern ""))
+                     helm-pattern)
+                "Enter a contact name to record")))
+    :action (lambda (candidate)
+              (addressbook-bookmark-set-1 candidate))))
+
 ;;; Transformer
 ;;
 
@@ -661,6 +695,16 @@ than `w3m-browse-url' use it."
                                          (file-directory-p isfile))))
                            (propertize trunc 'face 'helm-bookmark-directory
                                        'help-echo isfile))
+                          ( ;; Non existing files.
+                           (and isfile
+                                ;; Be safe and call `file-exists-p'
+                                ;; only if file is not remote or
+                                ;; remote but connected.
+                                (or (and (file-remote-p isfile)
+                                         (not (file-remote-p isfile nil t)))
+                                    (not (file-exists-p isfile))))
+                           (propertize trunc 'face 'helm-bookmark-file-not-found
+                                       'help-echo isfile))
                           ( ;; regular files
                            t
                            (propertize trunc 'face 'helm-bookmark-file
@@ -804,7 +848,8 @@ only if external addressbook-bookmark package is installed."
   "Preconfigured helm for addressbook bookmarks.
 Need addressbook-bookmark package as dependencie."
   (interactive)
-  (helm :sources 'helm-source-bookmark-addressbook
+  (helm :sources '(helm-source-bookmark-addressbook
+                   helm-source-addressbook-set)
         :prompt "Search Contact: "
         :buffer "*helm addressbook*"
         :default (list (thing-at-point 'symbol)
