@@ -51,7 +51,8 @@
 (declare-function async-byte-recompile-directory "ext:async-bytecomp.el")
 
 (defun helm-el-package--init ()
-  (let (package-menu-async)
+  (let (package-menu-async
+        (inhibit-read-only t))
     (when (null package-alist)
       (setq helm-el-package--show-only 'all))
     (when (and (fboundp 'package--removable-packages)
@@ -76,6 +77,8 @@
                'global
              (with-current-buffer (get-buffer "*Packages*")
                (setq helm-el-package--tabulated-list tabulated-list-entries)
+               (remove-text-properties (point-min) (point-max)
+                                       '(read-only button follow-link category))
                (buffer-string)))
            (setq helm-el-package--upgrades (helm-el-package-menu--find-upgrades))
            (if helm--force-updating-p
@@ -358,7 +361,8 @@
    (update :initform 'helm-el-package--update)
    (candidate-number-limit :initform 9999)
    (action :initform '(("Describe package" . helm-el-package-describe)
-                       ("Visit homepage" . helm-el-package-visit-homepage)))))
+                       ("Visit homepage" . helm-el-package-visit-homepage)))
+   (group :initform 'helm-el-package)))
 
 (defun helm-el-package--action-transformer (actions candidate)
   (let* ((pkg-desc (get-text-property 0 'tabulated-list-id candidate))
@@ -373,9 +377,16 @@
                    actions)))
     (cond (built-in '(("Describe package" . helm-el-package-describe)))
           ((and (package-installed-p pkg-name)
-                (cdr (assq pkg-name helm-el-package--upgrades)))
+                (cdr (assq pkg-name helm-el-package--upgrades))
+                (member status '("installed" "dependency")))
            (append '(("Upgrade package(s)" . helm-el-package-upgrade)
-                     ("Uninstall package(s)" . helm-el-package-uninstall)) acts))
+                     ("Uninstall package(s)" . helm-el-package-uninstall))
+                   acts))
+          ((and (package-installed-p pkg-name)
+                (cdr (assq pkg-name helm-el-package--upgrades))
+                (string= status "available"))
+           (append '(("Upgrade package(s)" . helm-el-package-upgrade))
+                   acts))
           ((and (package-installed-p pkg-name)
                 (or (null (package-built-in-p pkg-name))
                     (and (package-built-in-p pkg-name)
@@ -439,11 +450,13 @@
         :buffer "*helm list packages*"))
 
 ;;;###autoload
-(defun helm-list-elisp-packages-no-fetch ()
+(defun helm-list-elisp-packages-no-fetch (arg)
   "Preconfigured helm for emacs packages.
-Same as `helm-list-elisp-packages' but don't fetch packages on remote."
-  (interactive)
-  (let ((helm-el-package--initialized-p t))
+
+Same as `helm-list-elisp-packages' but don't fetch packages on remote.
+Called with a prefix ARG always fetch packages on remote."
+  (interactive "P")
+  (let ((helm-el-package--initialized-p (null arg)))
     (helm-list-elisp-packages nil)))
 
 (provide 'helm-elisp-package)
