@@ -426,66 +426,111 @@ The directory selection with \"**/\" like bash shopt globstar option is not supp
 
 *** Query replace regexp on filenames
 
-WARNING: This is designed to work ONLY in current directory, i.e
-         your marked files have to be from the same directory.
-         So do not mark files in different directories, [[Using wildcard to select multiple files][recursive globbing]] e.g \"**.txt\"
-         is not supported as well for same reasons. 
+Allow replacing different parts of file's basename with something else.
 
-You can rename your marked files by replacing only part of filenames matching
-a regexp.
+When calling this action you will be prompted like in `query-replace' with two prompts,
+one for the matching part of text to replace and another one for the replacement text,
+however several facilities are provided to make the two prompts more powerfull.
 
-e.g Rename recursively all files with \".JPG\" extension to \".jpg\":
-Use the helm-file-globstar feature described in previous section by
-entering at end of helm-find-files pattern \"**.JPG\", then hit `M-%`,
-at first prompt enter \"JPG\", at second \"jpg\" and hit `RET`.
+**** Syntax available in first prompt:
 
-Shortcut for basename without extension, only extension or all are available:
+In addition to a simple regexp, these shortcuts are availables:
 
 - Basename without extension => \"%.\"
 - Only extension             => \".%\"
-- All                        => \"%\"
+- Substring                  => \"%:<from>:<to>\"
+- Whole basename             => \"%\"
+
+**** Syntax available in second prompt
+
+In addition to a simple string to use as replacement, here what you can use:
+
+- A placeholder refering to what you have selected in first prompt: \"\\@\"
+After this placeholder you can use a search and replace syntax ala sed:
+
+    \"\\@/<regexp>/<replacement>/
+
+You can remove substring part of string represented by placeholder:
+
+    \"\\@:<from>:<to>\"
+ 
+- A special character representing a number which is incremented:   \"\\#\"
+
+- shortcut for `upcase', `downcase' and `capitalize'
+are available, respectively `%u', `%d' and `%c'.
+
+**** Usage with examples
+
+***** Rename recursively all files with \".JPG\" extension to \".jpg\":
+
+Use the `helm-file-globstar' feature described in [[Using wildcard to select multiple files][recursive globbing]] by
+entering at end of helm-find-files pattern \"**.JPG\", then hit \\<helm-map>\\[helm-ff-query-replace-on-filenames],
+at first prompt enter \"JPG\", at second \"jpg\" and hit `RET`.
 
 So in the example above you could do instead:
 At first prompt enter \".%\", at second \"jpg\" and hit `RET`.
+
 Note that when using this instead of using \"JPG\" at first prompt, all extensions
 will be renamed to \"jpg\" even if the extension of one of the files is e.g \"png\".
+If you want to keep the original extension you can use at first prompt \".%\",
+and at second \"%d\" (downcase).
 
-If you want to rename a serie of files from number 001 to 00x use \\# inside the replacement
-string when you will be prompted for it.
+***** Rename a serie of files from number 001 to 00x
+
+Use \\# inside the second prompt
 
 e.g To rename the files \"foo.jpg\" \"bar.jpg\" and \"baz.jpg\"
     to \"foo-001.jpg\" \"foo-002.jpg\" \"foo-003.jpg\"
 
 Use as replace regexp \"%.\" and as replacement string \"foo-\\#\".
 
-When \"%\", \".%\" or \"%\" are used, \"\\@\" can be used as a placeholder which
-remember those values.
-
 e.g To rename the files \"foo.jpg\" \"bar.jpg\" and \"baz.jpg\"
     to \"foo-001.jpg\" \"bar-002.jpg\" \"baz-003.jpg\"
 
 Use as replace regexp \"%.\" and as replacement string \"\\@-\\#\".
 
-Modifying the placeholder (\\@) is possible
-\(in contrast of renaming the whole placeholder with something else) with two methods:
+***** Replace a range of text
+
+    \"%:<from>:<to>\"
+
+e.g To rename files \"foo.jpg\" \"bar.jpg\" and \"baz.jpg\"
+    to \"fOo.jpg\" \"bAr.jpg\" and \"bAz.jpg\"
+
+Use as replace regexp \"%:1:2\" and as replacement string \"%u\" (aka upcase).
+
+NOTE that you CANNOT use \"%.\" and \".%\" along with substring replacement.
+
+***** Modifying string from the placeholder (\\@)
 
 - By substring, i.e using only the substring of placeholder:
     \\@:<from>:<to>
   e.g \\@:0:2 replaces from beginning to second char of placeholder
   Note that length of placeholder is used for <to> when <to> is not specified
   e.g \\@:2: replaces from second char of placeholder to end
+  This is a quick way to strip out one part of string.
 
 - By search and replace:
-    \\@/<regexp>/<replacement>
-  e.g \\@/foo/bar replaces \"foo\" in placeholder by \"bar\"
+    \\@/<regexp>/<replacement>/
+  e.g \\@/foo/bar/ replaces \"foo\" in placeholder by \"bar\"
   Incremental replacement is also handled in <replacement>
-  e.g \\@/foo/-\\# replaces \"foo\" in placeholder by 001, 002 etc...
+  e.g \\@/foo/-\\#/ replaces \"foo\" in placeholder by 001, 002 etc...
 
-In the second prompt (replace regexp with) shortcut for `upcase', `downcase' and `capitalize'
-are available, respectively `%u', `%d' and `%c'.
+***** Clash in replacements (avoid overwriting files)
 
-Note also that unlike the [[Serial renaming][serial rename]] actions the renamed files stay in their initial directory
-and are not renamed to current directory, IOW use this (\\#) to rename files inside the same directory.
+In all these replacements you may endup with same names as replacement, in such cases
+helm takes care of numbering the files that would overwrite precedent file, e.g:
+Say you remove in files \"emacs-m1.txt\" \"emacs-m2.txt\" and \"emacs-m3.txt\" the \"-m<n>\" part
+you would endup with three files named \"emacs.txt\", the second renaming overwriting first file,
+and the third renaming overwriting second file and so on, instead helm will rename like
+\"emacs(1).txt\" and \"emacs(2).txt\" second an third file.
+
+***** Query-replace on filenames vs serial rename actions
+
+Note also that unlike the [[Serial renaming][serial rename]] actions
+the renamed files stay in their initial directory and are not renamed
+to current directory, so usage of \\# to serial rename files make
+sense only for files inside the same directory even it continues renaming
+files with an incremental number in next directories.
 
 *** Serial renaming
 
@@ -507,6 +552,12 @@ Note that wildcards are supported as well, so you can use e.g \"*.txt\" to selec
 in current directory or \"**.txt\" to select all files recursively from current directory
 \(See [[Using wildcard to select multiple files]] section above).
 
+*** Defining default target directory for copying/renaming etc.. according to `helm-dwim-target'
+
+You can customize `helm-dwim-target' to behave differently according to the windows open
+in your current frame.
+Default is to provide completion on the directories belonging to these windows if some.
+
 *** Copying renaming asynchronously
 
 If you use async library (if you have installed helm from MELPA you do) you can enable
@@ -516,7 +567,7 @@ Note that even when async is enabled, running a copy/rename action with a prefix
 will execute action synchronously, it will follow also the first file of the marked files
 in its destination directory.
 
-When `dired-async-mode' is enabled and additional action named \"Backup files\" will be
+When `dired-async-mode' is enabled, an additional action named \"Backup files\" will be
 available (such command is not available in emacs natively).
 See [[Using wildcard to select multiple files]] for details.
 
@@ -1330,18 +1381,22 @@ but this will not push a new entry with concatenated candidates in kill-ring.
 
 *** Refiling
 
-The heading to refile will be the one you were at when starting helm
-session, and the place to refile this heading will be the selected
-candidate (i.e the candidate at point in helm buffer). If you want to
-refile another one, move to it in helm buffer, mark it, then move to
-the candidate of your choice to refile at this place.
-NOTE that of course if you have marked more than one candidate,
-all the subsequent candidates will be ignored.
+You can refile one heading or multiple headings at a time.
+
+To refile one heading, move the point to the entry you want to
+refile, and run \\[helm-org-in-buffer-headings].  Then select the
+heading you want to refile to, and press \\[C-c w], or select the
+refile action from the actions menu.
+
+To refile multiple headings, run \\[helm-org-in-buffer-headings]
+and mark the headings you want to refile.  Then select the
+heading you want to refile to (without marking it), and press
+\\[C-c w], or select the refile action from the actions menu.
 
 ** Commands
 \\<helm-org-headings-map>
 \\[helm-org-run-open-heading-in-indirect-buffer]\t\tOpen heading in indirect buffer.
-\\[helm-org-run-heading-refile]\t\tRefile current heading to selection.
+\\[helm-org-run-refile-heading-to]\t\tRefile current heading to selection, or marked headings to selection.
 \\[helm-org-run-insert-link-to-heading-at-marker]\t\tInsert link at point to selection."
   )
 
