@@ -20,6 +20,10 @@
 (require 'helm)
 (require 'helm-utils)
 (require 'org)
+
+;; Load org-with-point-at macro when compiling
+(eval-when-compile
+  (require 'org-macs))
 
 (declare-function org-agenda-switch-to "org-agenda.el")
 
@@ -58,7 +62,7 @@ Note this have no effect in `helm-org-in-buffer-headings'."
 (defcustom helm-org-headings-actions
   '(("Go to heading" . helm-org-goto-marker)
     ("Open in indirect buffer `C-c i'" . helm-org--open-heading-in-indirect-buffer)
-    ("Refile current or marked heading to selection `C-c w`" . helm-org-heading-refile)
+    ("Refile heading(s) (multiple-marked-to-selected, or current-to-selected) `C-c w`" . helm-org--refile-heading-to)
     ("Insert link to this heading `C-c l`" . helm-org-insert-link-to-heading-at-marker))
   "Default actions alist for
   `helm-source-org-headings-for-files'."
@@ -70,6 +74,12 @@ Note this have no effect in `helm-org-in-buffer-headings'."
   :type 'boolean
   :group 'helm-org)
 
+(defcustom helm-org-ignore-autosaves nil
+  "Ignore autosave files when starting `helm-org-agenda-files-headings'."
+  :type 'boolean
+  :group 'helm-org)
+
+
 ;;; Org capture templates
 ;;
 ;;
@@ -80,7 +90,7 @@ Note this have no effect in `helm-org-in-buffer-headings'."
                          collect (cons (nth 1 template) (nth 0 template)))
     :action '(("Do capture" . (lambda (template-shortcut)
                                 (org-capture nil template-shortcut))))))
-
+
 ;;; Org headings
 ;;
 ;;
@@ -111,7 +121,7 @@ Note this have no effect in `helm-org-in-buffer-headings'."
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map helm-map)
     (define-key map (kbd "C-c i") 'helm-org-run-open-heading-in-indirect-buffer)
-    (define-key map (kbd "C-c w") 'helm-org-run-heading-refile)
+    (define-key map (kbd "C-c w") 'helm-org-run-refile-heading-to)
     (define-key map (kbd "C-c l") 'helm-org-run-insert-link-to-heading-at-marker)
     map)
   "Keymap for `helm-source-org-headings-for-files'.")
@@ -204,7 +214,7 @@ nothing to CANDIDATES."
 (defun helm-org--get-candidates-in-file (filename &optional fontify nofname parents)
   (with-current-buffer (pcase filename
                          ((pred bufferp) filename)
-                         ((pred stringp) (find-file-noselect filename)))
+                         ((pred stringp) (find-file-noselect filename t)))
     (let ((match-fn (if fontify
                         #'match-string
                       #'match-string-no-properties))
@@ -275,28 +285,25 @@ nothing to CANDIDATES."
     (helm-exit-and-execute-action
      'helm-org-insert-link-to-heading-at-marker)))
 
-(defun helm-org-heading-refile (marker)
-  "Refile current heading or marked to MARKER.
-The current heading is the heading where cursor was
-before entering helm session, it will be used unless
-you mark a candidate, in this case helm will go to this marked
-candidate in org buffer and refile this candidate to MARKER.
-NOTE that of course if you have marked more than one candidate,
-all the subsequent candidates will be ignored."
-  (let ((mkd (with-helm-buffer
-               (and helm-marked-candidates
-                    (car (helm-marked-candidates))))))
-    (save-selected-window
-      (when (eq major-mode 'org-agenda-mode)
-        (org-agenda-switch-to))
-      (when mkd (helm-org-goto-marker mkd))
-      (org-cut-subtree)
-      (let ((target-level (with-current-buffer (marker-buffer marker)
-                            (goto-char (marker-position marker))
-                            (org-current-level))))
-        (helm-org-goto-marker marker)
-        (org-end-of-subtree t t)
-        (org-paste-subtree (+ target-level 1))))))
+(defun helm-org--refile-heading-to (marker)
+  "Refile headings to heading at MARKER.
+If multiple candidates are marked in the Helm session, they will
+all be refiled.  If no headings are marked, the selected heading
+will be refiled."
+  (let* ((victims (with-helm-buffer (helm-marked-candidates)))
+         (buffer (marker-buffer marker))
+         (filename (buffer-file-name buffer))
+         (rfloc (list nil filename nil marker)))
+    (when (and (= 1 (length victims))
+               (equal (helm-get-selection) (car victims)))
+      ;; No candidates are marked; we are refiling the entry at point
+      ;; to the selected heading
+      (setq victims (list (point))))
+    ;; Probably best to check that everything returned a value
+    (when (and victims buffer filename rfloc)
+      (cl-loop for victim in victims
+               do (org-with-point-at victim
+                    (org-refile nil nil rfloc))))))
 
 (defun helm-org-in-buffer-preselect ()
   (if (org-on-heading-p)
@@ -305,26 +312,36 @@ all the subsequent candidates will be ignored."
         (outline-previous-visible-heading 1)
         (buffer-substring-no-properties (point-at-bol) (point-at-eol)))))
 
-(defun helm-org-run-heading-refile ()
+(defun helm-org-run-refile-heading-to ()
   (interactive)
   (with-helm-alive-p
-    (helm-exit-and-execute-action 'helm-org-heading-refile)))
-(put 'helm-org-run-heading-refile 'helm-only t)
-
+    (helm-exit-and-execute-action 'helm-org--refile-heading-to)))
+(put 'helm-org-run-refile-heading-to 'helm-only t)
+
 ;;;###autoload
 (defun helm-org-agenda-files-headings ()
   "Preconfigured helm for org files headings."
   (interactive)
-  (helm :sources (helm-source-org-headings-for-files (org-agenda-files))
-        :candidate-number-limit 99999
-        :truncate-lines helm-org-truncate-lines
-        :buffer "*helm org headings*"))
+  (let ((autosaves (cl-loop for f in (org-agenda-files)
+                            when (file-exists-p
+                                  (expand-file-name
+                                   (concat "#" (helm-basename f) "#")
+                                   (helm-basedir f)))
+                            collect (helm-basename f))))
+    (when (or (null autosaves)
+              helm-org-ignore-autosaves
+              (y-or-n-p (format "%s have auto save data, continue?"
+                                (mapconcat 'identity autosaves ", "))))
+      (helm :sources (helm-source-org-headings-for-files (org-agenda-files))
+            :candidate-number-limit 99999
+            :truncate-lines helm-org-truncate-lines
+            :buffer "*helm org headings*"))))
 
 ;;;###autoload
 (defun helm-org-in-buffer-headings ()
   "Preconfigured helm for org buffer headings."
   (interactive)
-  (let (helm-org-show-filename helm-org-format-outline-path)
+  (let (helm-org-show-filename)
     (helm :sources (helm-source-org-headings-for-files
                     (list (current-buffer)))
           :candidate-number-limit 99999
@@ -354,7 +371,7 @@ current heading."
         :candidate-number-limit 99999
         :truncate-lines helm-org-truncate-lines
         :buffer "*helm org capture templates*"))
-
+
 ;;; Org tag completion
 
 ;; Based on code from Anders Johansson posted on 3 Mar 2016 at
