@@ -37,10 +37,12 @@
 
 (defcustom helm-kill-ring-max-offset 400
   "Max number of chars displayed per candidate in kill-ring browser.
-If nil, don't truncate candidate, show all.
+When `t', don't truncate candidate, show all.
 By default it is approximatively the number of bits contained in five lines
-of 80 chars each i.e 80*5."
-  :type '(choice (const :tag "Disabled" nil)
+of 80 chars each i.e 80*5.
+Note that if you set this to nil multiline will be disabled, i.e you
+will not have anymore separators between candidates."
+  :type '(choice (const :tag "Disabled" t)
           (integer :tag "Max candidate offset"))
   :group 'helm-ring)
 
@@ -51,7 +53,8 @@ of 80 chars each i.e 80*5."
 
 (defcustom helm-kill-ring-actions
   '(("Yank" . helm-kill-ring-action-yank)
-    ("Delete" . helm-kill-ring-action-delete))
+    ("Delete" . helm-kill-ring-action-delete)
+    ("Append" . helm-kill-ring-append))
   "List of actions for kill ring source."
   :group 'helm-ring
   :type '(alist :key-type string :value-type function))
@@ -63,24 +66,30 @@ of 80 chars each i.e 80*5."
 (defvar helm-kill-ring-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map helm-map)
-    (define-key map (kbd "M-y") 'helm-next-line)
-    (define-key map (kbd "M-u") 'helm-previous-line)
-    (define-key map (kbd "M-D") 'helm-kill-ring-delete)
-    (define-key map (kbd "C-]") 'helm-kill-ring-toggle-truncated)
+    (define-key map (kbd "M-y")     'helm-next-line)
+    (define-key map (kbd "M-u")     'helm-previous-line)
+    (define-key map (kbd "M-D")     'helm-kill-ring-delete)
+    (define-key map (kbd "C-M-w")   'helm-kill-ring-run-append)
+    (define-key map (kbd "C-]")     'helm-kill-ring-toggle-truncated)
+    (define-key map (kbd "C-c C-k") 'helm-kill-ring-kill-selection)
     map)
   "Keymap for `helm-show-kill-ring'.")
 
 (defvar helm-source-kill-ring
   (helm-build-sync-source "Kill Ring"
-    :init (lambda () (helm-attrset 'last-command last-command))
+    :init (lambda ()
+            (helm-attrset 'last-command last-command)
+            (helm-attrset 'multiline helm-kill-ring-max-offset))
     :candidates #'helm-kill-ring-candidates
     :filtered-candidate-transformer #'helm-kill-ring-transformer
     :action 'helm-kill-ring-actions
     :persistent-action 'ignore
+    :help-message 'helm-kill-ring-help-message
     :persistent-help "DoNothing"
     :keymap helm-kill-ring-map
     :migemo t
-    :multiline t)
+    :multiline 'helm-kill-ring-max-offset
+    :group 'helm-ring)
   "Source for browse and insert contents of kill-ring.")
 
 (defun helm-kill-ring-candidates ()
@@ -90,30 +99,11 @@ of 80 chars each i.e 80*5."
         collect kill))
 
 (defun helm-kill-ring-transformer (candidates _source)
-  "Display only the `helm-kill-ring-max-lines-number' lines of candidate."
+  "Ensure CANDIDATES are not read-only."
   (cl-loop for i in candidates
            when (get-text-property 0 'read-only i)
            do (set-text-properties 0 (length i) '(read-only nil) i)
-           collect (cons (helm-kill-ring--get-truncated-candidate i) i)))
-
-(defun helm-kill-ring--get-truncated-candidate (candidate)
-  "Truncate CANDIDATE when its length is > than `helm-kill-ring-max-offset'."
-  (with-temp-buffer
-    (insert candidate)
-    (goto-char (point-min))
-    (if (and helm-kill-ring-max-offset
-             (> (buffer-size) helm-kill-ring-max-offset))
-        (let ((end-str "[...]"))
-          (concat
-           (buffer-substring
-            (point)
-            (save-excursion
-              (forward-char helm-kill-ring-max-offset)
-              (setq end-str (if (looking-at "\n")
-                                end-str (concat "\n" end-str)))
-              (point)))
-           end-str))
-        (buffer-string))))
+           collect i))
 
 (defvar helm-kill-ring--truncated-flag nil)
 (defun helm-kill-ring-toggle-truncated ()
@@ -124,14 +114,20 @@ of 80 chars each i.e 80*5."
     (let* ((cur-cand (helm-get-selection))
            (presel-fn (lambda ()
                         (helm-kill-ring--preselect-fn cur-cand))))
-      (let ((helm-kill-ring-max-offset
-             (if helm-kill-ring--truncated-flag
-                 15000000
-                 (default-toplevel-value
-                     'helm-kill-ring-max-offset))))
-        (helm-update presel-fn)))))
+      (helm-attrset 'multiline
+                    (if helm-kill-ring--truncated-flag
+                        15000000
+                        helm-kill-ring-max-offset))
+        (helm-update presel-fn))))
 (put 'helm-kill-ring-toggle-truncated 'helm-only t)
 
+(defun helm-kill-ring-kill-selection ()
+  "Store the real value of candidate in kill-ring.
+Same as `helm-kill-selection-and-quit' called with a prefix arg."
+  (interactive)
+  (helm-kill-selection-and-quit t))
+(put 'helm-kill-ring-kill-selection 'helm-only t)
+
 (defun helm-kill-ring--preselect-fn (candidate)
   "Internal, used to preselect CANDIDATE when toggling truncated view."
   ;; Preselection by regexp may not work if candidate is huge, so walk
@@ -149,34 +145,36 @@ of 80 chars each i.e 80*5."
 If this action is executed just after `yank',
 replace with STR as yanked string."
   (with-helm-current-buffer
-    (setq kill-ring (delete str kill-ring))
-    ;; Adding a `delete-selection' property
-    ;; to `helm-kill-ring-action' is not working
-    ;; because `this-command' will be `helm-maybe-exit-minibuffer',
-    ;; so use this workaround (Issue #1520).
-    (when (and (region-active-p) delete-selection-mode)
-      (delete-region (region-beginning) (region-end)))
-    (if (not (eq (helm-attr 'last-command helm-source-kill-ring) 'yank))
-        (insert-for-yank str)
-      ;; from `yank-pop'
-      (let ((inhibit-read-only t)
-            (before (< (point) (mark t))))
-        (if before
-            (funcall (or yank-undo-function 'delete-region) (point) (mark t))
-          (funcall (or yank-undo-function 'delete-region) (mark t) (point)))
-        (setq yank-undo-function nil)
-        (set-marker (mark-marker) (point) helm-current-buffer)
-        (insert-for-yank str)
-        ;; Set the window start back where it was in the yank command,
-        ;; if possible.
-        (set-window-start (selected-window) yank-window-start t)
-        (when before
-          ;; This is like exchange-point-and-mark, but doesn't activate the mark.
-          ;; It is cleaner to avoid activation, even though the command
-          ;; loop would deactivate the mark because we inserted text.
-          (goto-char (prog1 (mark t)
-                       (set-marker (mark-marker) (point) helm-current-buffer))))))
-    (kill-new str)))
+    (unwind-protect
+         (progn
+           (setq kill-ring (delete str kill-ring))
+           ;; Adding a `delete-selection' property
+           ;; to `helm-kill-ring-action' is not working
+           ;; because `this-command' will be `helm-maybe-exit-minibuffer',
+           ;; so use this workaround (Issue #1520).
+           (when (and (region-active-p) delete-selection-mode)
+             (delete-region (region-beginning) (region-end)))
+           (if (not (eq (helm-attr 'last-command helm-source-kill-ring) 'yank))
+               (insert-for-yank str)
+               ;; from `yank-pop'
+               (let ((inhibit-read-only t)
+                     (before (< (point) (mark t))))
+                 (if before
+                     (funcall (or yank-undo-function 'delete-region) (point) (mark t))
+                     (funcall (or yank-undo-function 'delete-region) (mark t) (point)))
+                 (setq yank-undo-function nil)
+                 (set-marker (mark-marker) (point) helm-current-buffer)
+                 (insert-for-yank str)
+                 ;; Set the window start back where it was in the yank command,
+                 ;; if possible.
+                 (set-window-start (selected-window) yank-window-start t)
+                 (when before
+                   ;; This is like exchange-point-and-mark, but doesn't activate the mark.
+                   ;; It is cleaner to avoid activation, even though the command
+                   ;; loop would deactivate the mark because we inserted text.
+                   (goto-char (prog1 (mark t)
+                                (set-marker (mark-marker) (point) helm-current-buffer)))))))
+      (kill-new str))))
 (define-obsolete-function-alias 'helm-kill-ring-action 'helm-kill-ring-action-yank "2.4.0")
 
 (defun helm-kill-ring-action-delete (_candidate)
@@ -193,6 +191,19 @@ This is a command for `helm-kill-ring-map'."
   (with-helm-alive-p
     (helm-exit-and-execute-action 'helm-kill-ring-action-delete)))
 
+(defun helm-kill-ring-append (_candidate)
+  "Yank concatenated marked candidates."
+  (let ((marked (helm-marked-candidates)))
+    (helm-kill-ring-action-yank
+     (cl-loop for cand in marked
+              for sep = (if (string-match "\n\\'" cand) "" "\n")
+              concat (concat cand sep)))))
+
+(defun helm-kill-ring-run-append ()
+  "Yank concatenated marked candidates."
+  (interactive)
+  (with-helm-alive-p
+    (helm-exit-and-execute-action 'helm-kill-ring-append)))
 
 ;;;; <Mark ring>
 ;; DO NOT use these sources with other sources use
@@ -205,6 +216,7 @@ This is a command for `helm-kill-ring-map'."
     (goto-char pos)
     (forward-line 0)
     (let ((line (car (split-string (thing-at-point 'line) "[\n\r]"))))
+      (remove-text-properties 0 (length line) '(read-only) line)
       (if (string= "" line)
           "<EMPTY LINE>"
         line))))
@@ -212,41 +224,40 @@ This is a command for `helm-kill-ring-map'."
 (defun helm-mark-ring-get-candidates ()
   (with-helm-current-buffer
     (cl-loop with marks = (if (mark t) (cons (mark-marker) mark-ring) mark-ring)
-             for i in marks
+             for marker in marks
              with max-line-number = (line-number-at-pos (point-max))
              with width = (length (number-to-string max-line-number))
              for m = (format (concat "%" (number-to-string width) "d: %s")
-                             (line-number-at-pos i)
-                             (helm-mark-ring-line-string-at-pos i))
-             unless (and recip (member m recip))
-             collect m into recip
+                             (line-number-at-pos marker)
+                             (helm-mark-ring-line-string-at-pos marker))
+             unless (and recip (assoc m recip))
+             collect (cons m marker) into recip
              finally return recip)))
 
+(defun helm-mark-ring-default-action (candidate)
+  (let ((target (copy-marker candidate)))
+    (switch-to-buffer (marker-buffer candidate))
+    (helm-log-run-hook 'helm-goto-line-before-hook)
+    (helm-match-line-cleanup)
+    (with-helm-current-buffer
+      (unless helm-yank-point (setq helm-yank-point (point))))
+    (helm-goto-char target)
+    (helm-highlight-current-line)))
+
 (defvar helm-source-mark-ring
   (helm-build-sync-source "mark-ring"
     :candidates #'helm-mark-ring-get-candidates
-    :action '(("Goto line"
-               . (lambda (candidate)
-                   (helm-goto-line (string-to-number candidate))))) 
-    :persistent-action (lambda (candidate)
-                         (switch-to-buffer helm-current-buffer)
-                         (helm-goto-line (string-to-number candidate)))
-    :persistent-help "Show this line"))
+    :action '(("Goto line" . helm-mark-ring-default-action)) 
+    :persistent-help "Show this line"
+    :group 'helm-ring))
 
 ;;; Global-mark-ring
 (defvar helm-source-global-mark-ring
   (helm-build-sync-source "global-mark-ring"
     :candidates #'helm-global-mark-ring-get-candidates
-    :action '(("Goto line"
-              . (lambda (candidate)
-                  (let ((items (split-string candidate ":")))
-                    (switch-to-buffer (cl-second items))
-                    (helm-goto-line (string-to-number (car items)))))))
-    :persistent-action (lambda (candidate)
-                         (let ((items (split-string candidate ":")))
-                           (switch-to-buffer (cl-second items))
-                           (helm-goto-line (string-to-number (car items)))))
-    :persistent-help "Show this line"))
+    :action '(("Goto line" . helm-mark-ring-default-action))
+    :persistent-help "Show this line"
+    :group 'helm-ring))
 
 (defun helm-global-mark-ring-format-buffer (marker)
   (with-current-buffer (marker-buffer marker)
@@ -257,19 +268,20 @@ This is a command for `helm-kill-ring-map'."
                         (guard (not (string-match-p "\\`\n?\\'" line))))
                    (car (split-string line "[\n\r]")))
                   (_ "<EMPTY LINE>"))))
+      (remove-text-properties 0 (length line) '(read-only) line)
       (format "%7d:%s:    %s"
               (line-number-at-pos) (marker-buffer marker) line))))
 
 (defun helm-global-mark-ring-get-candidates ()
   (let ((marks global-mark-ring))
     (when marks
-      (cl-loop for i in marks
-               for mb = (marker-buffer i)
+      (cl-loop for marker in marks
+               for mb = (marker-buffer marker)
                for gm = (unless (or (string-match "^ " (format "%s" mb))
                                     (null mb))
-                          (helm-global-mark-ring-format-buffer i))
-               when (and gm (not (member gm recip)))
-               collect gm into recip
+                          (helm-global-mark-ring-format-buffer marker))
+               when (and gm (not (assoc gm recip)))
+               collect (cons gm marker) into recip
                finally return recip))))
 
 (defun helm--push-mark (&optional location nomsg activate)
@@ -323,7 +335,8 @@ the `global-mark-ring' after each new visit."
                           for register = (car candidate)
                           do (setq register-alist
                                 (delq (assoc register register-alist)
-                                      register-alist)))))))
+                                      register-alist))))))
+    :group 'helm-ring)
   "See (info \"(emacs)Registers\")")
 
 (defun helm-register-candidates ()
@@ -395,35 +408,33 @@ the `global-mark-ring' after each new visit."
 
 (defun helm-register-action-transformer (actions register-and-functions)
   "Decide actions by the contents of register."
-  (cl-loop with transformer-actions = nil
-           with func-actions =
-        '((insert-register
-           "Insert Register" .
-           (lambda (c) (insert-register (car c))))
-          (jump-to-register
-           "Jump to Register" .
-           (lambda (c) (jump-to-register (car c))))
-          (append-to-register
-           "Append Region to Register" .
-           (lambda (c) (append-to-register
-                        (car c) (region-beginning) (region-end))))
-          (prepend-to-register
-           "Prepend Region to Register" .
-           (lambda (c) (prepend-to-register
-                        (car c) (region-beginning) (region-end))))
-          (increment-register
-           "Increment Prefix Arg to Register" .
-           (lambda (c) (increment-register
-                        helm-current-prefix-arg (car c))))
-          (undo-tree-restore-state-from-register
-           "Restore Undo-tree register" .
-           (lambda (c) (and (fboundp 'undo-tree-restore-state-from-register)
-                            (undo-tree-restore-state-from-register (car c))))))
-        for func in (cdr register-and-functions)
-        for cell = (assq func func-actions)
-        when cell
-        do (push (cdr cell) transformer-actions)
-        finally return (append (nreverse transformer-actions) actions)))
+  (cl-loop with func-actions =
+           '((insert-register
+              "Insert Register" .
+              (lambda (c) (insert-register (car c))))
+             (jump-to-register
+              "Jump to Register" .
+              (lambda (c) (jump-to-register (car c))))
+             (append-to-register
+              "Append Region to Register" .
+              (lambda (c) (append-to-register
+                           (car c) (region-beginning) (region-end))))
+             (prepend-to-register
+              "Prepend Region to Register" .
+              (lambda (c) (prepend-to-register
+                           (car c) (region-beginning) (region-end))))
+             (increment-register
+              "Increment Prefix Arg to Register" .
+              (lambda (c) (increment-register
+                           helm-current-prefix-arg (car c))))
+             (undo-tree-restore-state-from-register
+              "Restore Undo-tree register" .
+              (lambda (c) (and (fboundp 'undo-tree-restore-state-from-register)
+                               (undo-tree-restore-state-from-register (car c))))))
+           for func in (cdr register-and-functions)
+           when (assq func func-actions)
+           collect (cdr it) into transformer-actions
+           finally return (append transformer-actions actions)))
 
 ;;;###autoload
 (defun helm-mark-ring ()
@@ -505,7 +516,8 @@ This command is useful when used with persistent action."
              (kmacro-push-ring)
              (kmacro-split-ring-element candidate)
              (kmacro-exec-ring-item
-              candidate helm-current-prefix-arg))))
+              candidate helm-current-prefix-arg)))
+          :group 'helm-ring)
         :buffer "*helm kmacro*"))
 
 (provide 'helm-ring)
