@@ -669,10 +669,12 @@ inherit from `helm-source'.")
     :custom function
     :documentation
     "  A function like `buffer-substring-no-properties' or `buffer-substring'.
-  This function converts point of line-beginning and point of line-end,
-  which represents a candidate computed by `helm-candidates-in-buffer'.
+  This function converts region from point at line-beginning and point
+  at line-end in the `helm-candidate-buffer' to a string which will be displayed
+  in the `helm-buffer', it takes two args BEG and END.
   By default, `helm-candidates-in-buffer' uses
-  `buffer-substring-no-properties'.")
+  `buffer-substring-no-properties' which does no conversion and doesn't carry
+  text properties.")
 
    (search
     :initarg :search
@@ -707,8 +709,15 @@ inherit from `helm-source'.")
         :MULTIMATCH slot."))
 
   "Use this source to make helm sources storing candidates inside a buffer.
+
+The buffer storing candidates is generated by `helm-candidate-buffer' function
+and all search are done in this buffer, results are transfered to the `helm-buffer'
+when done.
 Contrarily to `helm-source-sync' candidates are matched using a function
-like `re-search-forward', see below documentation of :search slot.
+like `re-search-forward' (see below documentation of `:search' slot) which makes
+the search much faster than matching candidates one by one.
+If you want to add search functions to your sources, don't use `:match' which
+will raise an error, but `:search'.
 See `helm-candidates-in-buffer' for more infos.")
 
 (defclass helm-source-dummy (helm-source)
@@ -908,9 +917,6 @@ an eieio class."
       (setf (slot-value source 'header-line)
             (helm-source--persistent-help-string it source))
     (setf (slot-value source 'header-line) (helm-source--header-line source)))
-  (helm-aif (slot-value source 'candidate-number-limit)
-      (and (symbolp it) (setf (slot-value source 'candidate-number-limit)
-                              (symbol-value it))))
   (when (and (slot-value source 'fuzzy-match) helm-fuzzy-sort-fn)
     (setf (slot-value source 'filtered-candidate-transformer)
           (helm-aif (slot-value source 'filtered-candidate-transformer)
